# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:    modules/Ncs.ycp
# Package:    Configuration of Cluster Services (NCS)
# Summary:    Cluster Services (NCS) settings, input and output functions
# Authors:    Michael G. Fritch <mgfritch@novell.com>
#
# $Id: Ncs.ycp,v 1.1.2.50 2005/10/10 19:43:37 rjorgensen Exp $
#
# Representation of the configuration of Cluster Services (NCS).
# Input and output routines.
require "yast"

module Yast
  class NcsClass < Module
    def main
      textdomain "novell-ncs"

      Yast.import "Popup"
      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Service"
      Yast.import "Summary"
      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "Misc"
      Yast.import "NovellUtils"
      Yast.import "Edirectory"
      Yast.import "EdirUtils"
      Yast.import "NovellLDAP"
      Yast.import "NovellPatterns"
      Yast.import "NovellAddonInstall"
      Yast.import "SuSEFirewall"
      Yast.import "Wizard"
      @auto_clone = false

      @Settings = {}
      @oldSettings = {}
      @sysconf_file = "/etc/sysconfig/novell/ncs_oes2sp1" #The /etc/sysconfig file.

      @xad_packages_installed = false
      @xadMode = false

      @xadTestMode = false

      @testMode = false

      @globals_configured = false

      @edir_service_configured = false


      # Are we in upgrade mode?
      @upgradeModeOES11SP2 = false # upgrading from OES 11 SP2
      @upgradeModeOES11SP3 = false # upgrading from OES 11 SP3
      @upgradeModeOES2015 = false
      @upgradeModeOES2015SP1 = false
      @upgradeModeOES2018 = false # upgrading from OES 2018
      @upgradeModeOES2018SP1 = false # upgrading from OES 2018 SP1
      @upgradeModeOES2018SP2 = false # upgrading from OES 2018 SP2

      # Data was modified?
      @modified = false


      @proposal_valid = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")

      # autoyast_tool, set to true only when the autoyast control file
      # generation tool is running
      @autoyast_tool = false
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # If this method returns false then the tree admin and password is obtained from the following map values:
    # Note: this method only works if eDir is installed on the local box.
    #
    #    NovellLDAP::Settings["CONFIG_LDAP_ADMIN_CONTEXT"]:""
    #    NovellLDAP::Settings["CONFIG_LDAP_ADMIN_PASSWORD"]:""
    #
    # If this method returns true then the tree admin and password is obtained from the following map values:
    #
    #
    #    NovellLDAP::Settings["XAD_TREE_ADMIN_CONTEXT"]:""
    #    NovellLDAP::Settings["XAD_TREE_ADMIN_PASSWORD"]:""
    #
    # @return true if tree admin is xad admin
    def UseXadLdapForTreeAdmin
      ret = false

      # if we are not doing ndsw, then we just use the usual admin.
      if @xadMode == true
        # First see if we are running with a new tree or an existing tree.
        treeType = ""
        if Mode.installation && Stage.normal && @edir_service_configured
          treeType = Convert.to_string(
            SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))
          ) # next try reading the edir conf file
          Builtins.y2milestone(
            "Ncs.UseXadLdapForTreeAdmin (post install):tree type from edir conf file:%1",
            treeType
          )
        else
          treeType = Ops.get_string(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_TREE_TYPE",
            ""
          )
          Builtins.y2milestone(
            "Ncs.UseXadLdapForTreeAdmin:tree type from eDir map:%1",
            treeType
          )
          if treeType == nil || treeType == ""
            treeType = Convert.to_string(
              SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))
            ) # next try reading the edir conf file
            Builtins.y2milestone(
              "Ncs.UseXadLdapForTreeAdmin:tree type from edir conf file:%1",
              treeType
            )
          end
        end
        if treeType == nil || treeType == ""
          treeType = Edirectory.ProposeTreeType # worst case, propose the tree type.
          Builtins.y2milestone(
            "Ncs.UseXadLdapForTreeAdmin:tree type from propose tree:%1",
            treeType
          )
        end
        ret = true if treeType == "existing"
      end

      ret
    end

    # Start NCS
    # @return true if started
    def StartNCS
      Builtins.y2milestone("NCS: starting cluster services")

      #Start the cluster services
      Service.Enable("novell-ncs")
      if Ops.get_string(@Settings, "CONFIG_NCS_START", "Now") == "Now"
        Service.Start("novell-ncs")
      end
      true
    end


    # Read all ncs settings
    # @return true on success
    def Read
      # Ncs read dialog caption
      caption = _("Initializing OES Cluster Services (NCS) Configuration")

      steps = 2
      sl = 500

      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Read OES Cluster Services (NCS) settings"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/1
          _("Reading OES Cluster Services (NCS) settings..."),
          _("Checking dependencies"),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read current settings
      return false if Abort()
      Progress.NextStage
      NovellUtils.AddtoOpenLdapConfFile

      if Builtins.setenv("LDAPTLS_REQCERT", "never")
        Builtins.y2milestone("LDAPTLS_REQCERT is set")
      else
        Builtins.y2milestone("failed to set LDAPTLS_REQCERT")
      end

      NovellUtils.LogInstallMode

      upgradeInfo = {}
      # See if we are doing an upgrade from OES 2018 SP2
      @upgradeModeOES2018SP2 = NovellAddonInstall.IsProductUpgradeOES2018SP2(
        "ncs"
      )
      if @upgradeModeOES2018SP2 == false
        # See if we are doing an upgrade from OES 2018 SP1
        @upgradeModeOES2018SP1 = NovellAddonInstall.IsProductUpgradeOES2018SP1(
          "ncs"
        )
        if @upgradeModeOES2018SP1 == false
          # See if we are doing an upgrade from OES 2018
          @upgradeModeOES2018 = NovellAddonInstall.IsProductUpgradeOES2018(
            "ncs"
          )
          if @upgradeModeOES2018 == false
            # See if we are doing an upgrade from OES 2015 SP1
            @upgradeModeOES2015SP1 = NovellAddonInstall.IsProductUpgradeOES2015SP1(
              "ncs"
            )
            if @upgradeModeOES2015SP1 == false
              # See if we are doing an upgrade from OES 11 SP3
              @upgradeModeOES11SP3 = NovellAddonInstall.IsProductUpgradeOES11SP3("ncs")
              if @upgradeModeOES11SP3 == false
                # See if we are doing an upgrade from OES 2015
                @upgradeModeOES2015 = NovellAddonInstall.IsProductUpgradeOES2015("ncs")
                if @upgradeModeOES2015 == false
                  # See if we are doing an upgrade from OES 11 SP2
                  @upgradeModeOES11SP2 = NovellAddonInstall.IsProductUpgradeOES11SP2(
                    "ncs"
                  )
                  if @upgradeModeOES11SP2
                    upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES11SP2(
                      "ncs"
                    )
                    @oldSettings = NovellAddonInstall.GetSysconfigMap(
                      Ops.get_string(upgradeInfo, "config_file", "")
                    )
                  end #upgrading from OES 2015
                else
                  upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2015("ncs")
                  @oldSettings = NovellAddonInstall.GetSysconfigMap(
                    Ops.get_string(upgradeInfo, "config_file", "")
                  )
                end #upgrading from OES 11 SP3
              else
                upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES11SP3("ncs")
                @oldSettings = NovellAddonInstall.GetSysconfigMap(
                  Ops.get_string(upgradeInfo, "config_file", "")
                )
              end
            else
              upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2015SP1(
                "ncs"
              )
              @oldSettings = NovellAddonInstall.GetSysconfigMap(
                Ops.get_string(upgradeInfo, "config_file", "")
              )
            end
          else
            upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018(
              "ncs"
            )
            @oldSettings = NovellAddonInstall.GetSysconfigMap(
              Ops.get_string(upgradeInfo, "config_file", "")
            )
          end
        else
          upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018SP1(
            "ncs"
          )
          @oldSettings = NovellAddonInstall.GetSysconfigMap(
            Ops.get_string(upgradeInfo, "config_file", "")
          )
        end
      else
        upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018SP2(
          "ncs"
        )
        @oldSettings = NovellAddonInstall.GetSysconfigMap(
          Ops.get_string(upgradeInfo, "config_file", "")
        )
      end
  
  
      Builtins.y2milestone(
        "ncs:
        upgradeModeOES11SP2 = #{@upgradeModeOES11SP2},
        upgradeModeOES2015 = #{@upgradeModeOES2015},
        upgradeModeOES2015SP1 = #{@upgradeModeOES2015SP1},
        upgradeModeOES11SP3 = #{@upgradeModeOES11SP3},
        upgradeModeOES2018 = #{@upgradeModeOES2018},
        upgradeModeOES2018SP1 = #{@upgradeModeOES2018SP1},
        upgradeModeOES2018SP2 = #{@upgradeModeOES2018SP2}"
      )
      Builtins.y2milestone("ncs:oldSettings: %1", @oldSettings)

      info = NovellAddonInstall.GetProductInfo("edirectory")
      if Ops.get_boolean(info, "SERVICE_CONFIGURED", false) == true
        @edir_service_configured = true
      else
        @edir_service_configured = false
      end

      # See if XAD is installed.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "Ncs:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end
      Builtins.y2milestone("Ncs:xadMode:%1", @xadMode)

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Read
      Progress.set(progress_orig)

      setDefaultIP = false

      # determine if NCS configuration already exists - if it does not, do the following...
      service_configured = Convert.to_string(
        SCR.Read(path(".sysconfig.ncs.SERVICE_CONFIGURED"))
      )
      service_configured = Builtins.tolower(service_configured)

      if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1 ||  @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 || @upgradeModeOES2018SP2
        Ops.set(
          @Settings,
          "CONFIG_NCS_LDAP_SERVERS",
          Builtins.splitstring(
            Ops.get_string(@oldSettings, "CONFIG_NCS_LDAP_SERVERS", ""),
            ","
          )
        )
        Ops.set(
          @Settings,
          "CONFIG_NCS_START",
          Ops.get_string(@oldSettings, "CONFIG_NCS_START", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_NCS_CONFIG_TYPE",
          Ops.get_string(@oldSettings, "CONFIG_NCS_CONFIG_TYPE", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_NCS_CLUSTER_DN",
          Ops.get_string(@oldSettings, "CONFIG_NCS_CLUSTER_DN", "")
        )
        if Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "New Cluster"
          Ops.set(
            @Settings,
            "CONFIG_NCS_CLUSTER_IP",
            Ops.get_string(@oldSettings, "CONFIG_NCS_CLUSTER_IP", "")
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_SBD_DEV",
            Ops.get_string(@oldSettings, "CONFIG_NCS_SBD_DEV", "")
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_SBD_DEV2",
            Ops.get_string(@oldSettings, "CONFIG_NCS_SBD_DEV2", "")
          )
        end

        Ops.set(
          @Settings,
          "CONFIG_NCS_SERVER_NAME",
          Ops.get_string(@oldSettings, "CONFIG_NCS_SERVER_NAME", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_NCS_SERVER_IP",
          Ops.get_string(@oldSettings, "CONFIG_NCS_SERVER_IP", "")
        )
        Ops.set(@Settings, "CONFIG_NCS_PROXY_USER", "")
        Ops.set(@Settings, "CONFIG_NCS_PROXY_PASSWORD", "")
        Ops.set(@Settings, "CONFIG_NCS_USE_COMMON_PROXY", "no")

        if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1 ||  @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 || @upgradeModeOES2018SP2
	    Ops.set(@Settings, "CONFIG_NCS_PROXY_USER",
            Ops.get_string(@oldSettings, "CONFIG_NCS_PROXY_USER", "")
            )
        end

      else
        useCommonProxy = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_DO_PROXY",
          ""
        )
        Builtins.y2milestone("Ncs.Read:useCommonProxy: %1", useCommonProxy)

        commonProxyName = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
        )
        Builtins.y2milestone(
          "Ncs.Read:commonProxyName: '%1'",
          commonProxyName
        )
        commonProxyPassword = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_PROXY_PASSWORD",
          ""
        )

        if useCommonProxy == "yes"
          if commonProxyName == ""
            commonProxyName = NovellUtils.RetrieveProxyCredentials(
              "username"
            )
          end
          if commonProxyPassword == ""
            commonProxyPassword = NovellUtils.RetrieveProxyCredentials(
              "password"
            )
          end
          Ops.set(@Settings, "CONFIG_NCS_COMMON_PROXY", commonProxyName)
          Ops.set(
            @Settings,
            "CONFIG_NCS_COMMON_PROXY_PASSWORD",
            commonProxyPassword
          )
        end

        adminDn = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        adminPassword = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )
        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "NovellLum:XAD_TREE_ADMIN_CONTEXT:%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            adminDn = NovellUtils.Nds2Ldap(xad_tree_admin_context)
            adminPassword = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              ""
            )
          end
        end
        Builtins.y2milestone("Ncs.Read:admin:%1", adminDn)

        if service_configured != "yes"
          Builtins.y2milestone("Ncs.Read:we are in new install read() section.")
          if useCommonProxy == "yes"
            Ops.set(@Settings, "CONFIG_NCS_USE_COMMON_PROXY", "yes")
            Ops.set(@Settings, "CONFIG_NCS_PROXY_USER", commonProxyName)
            Ops.set(@Settings, "CONFIG_NCS_PROXY_PASSWORD", commonProxyPassword)
          else
            Ops.set(@Settings, "CONFIG_NCS_USE_COMMON_PROXY", "no")
            Ops.set(@Settings, "CONFIG_NCS_PROXY_USER", adminDn) # default to LDAP admin
            Ops.set(@Settings, "CONFIG_NCS_PROXY_PASSWORD", adminPassword) # default to LDAP admin Password
          end

          setDefaultIP = true
          server_list = NovellLDAP.GetRemoteServers(nil)
          Ops.set(@Settings, "CONFIG_NCS_LDAP_SERVERS", [])
          Builtins.foreach(server_list) do |server_info|
            Ops.set(
              @Settings,
              "CONFIG_NCS_LDAP_SERVERS",
              Builtins.add(
                Ops.get_list(@Settings, "CONFIG_NCS_LDAP_SERVERS", []),
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
              )
            )
          end 

          server_list = NovellLDAP.GetLocalServers(nil)
          Builtins.foreach(server_list) do |server_info|
            Ops.set(
              @Settings,
              "CONFIG_NCS_LDAP_SERVERS",
              Builtins.add(
                Ops.get_list(@Settings, "CONFIG_NCS_LDAP_SERVERS", []),
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
              )
            )
          end 


          # Now, set up the data.
          adminContext = NovellUtils.GetContextNDS(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
          )
          Builtins.y2milestone("Ncs:adminContext is %1", adminContext)

          # First if in xad mode default the adminContext to the correct NDS tree context
          if @xadMode
            xad_tree_admin_context = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_CONTEXT",
              ""
            )
            Builtins.y2milestone(
              "Ncs:XAD_TREE_ADMIN_CONTEXT:%1",
              xad_tree_admin_context
            )
            if xad_tree_admin_context != nil && xad_tree_admin_context != ""
              adminContext = NovellUtils.GetContextNDS(xad_tree_admin_context)
            end
          end
          Builtins.y2milestone("Ncs:admin context:%1", adminContext)

          # Now that we have a default, try to use the server context if possible.
          #        if( xadMode )
          #	{
          wrkContext = ""
          if Mode.installation && Stage.normal && @edir_service_configured
            wrkContext = Misc.SysconfigRead(
              path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"),
              ""
            )
            Builtins.y2milestone(
              "Ncs:Read (post install mode).eDir server context from conf file: %1",
              wrkContext
            )
          else
            wrkContext = Ops.get_string(
              Edirectory.EdirSettings,
              "CONFIG_EDIR_SERVER_CONTEXT",
              ""
            )
            Builtins.y2milestone(
              "Ncs:Read.eDir server context from map: %1",
              wrkContext
            )
            if wrkContext == nil || wrkContext == ""
              wrkContext = Misc.SysconfigRead(
                path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"),
                ""
              )
              Builtins.y2milestone(
                "Ncs:Read.eDir server context from conf file: %1",
                wrkContext
              )
            end
          end
          Builtins.y2milestone("Ncs: wrkContext is %1", wrkContext)
          if wrkContext != nil && wrkContext != ""
            adminContext = NovellUtils.Nds2Ldap(wrkContext)
            Builtins.y2milestone("Ncs:adminContext is now %1", adminContext)
          end
          #	}

          Builtins.y2milestone("Ncs:adminContext now %1", adminContext)
          defaultClusterName = ""
          if adminContext != ""
            defaultClusterName = Ops.add(
              "cn=cluster,",
              NovellUtils.Nds2Ldap(adminContext)
            )
            Builtins.y2milestone(
              "Ncs:default cluster name is %1",
              defaultClusterName
            )
          end
          Ops.set(@Settings, "CONFIG_NCS_CONFIG_TYPE", "Existing Cluster")
          Ops.set(@Settings, "CONFIG_NCS_START", "Now")
          Ops.set(@Settings, "CONFIG_NCS_CLUSTER_DN", defaultClusterName)
          Ops.set(@Settings, "CONFIG_NCS_CLUSTER_IP", "")
          Ops.set(@Settings, "CONFIG_NCS_SBD_DEV", "")
          Ops.set(@Settings, "CONFIG_NCS_SBD_DEV2", "")
          Ops.set(@Settings, "CONFIG_NCS_SERVER_NAME", NovellUtils.GetHostname)
          Ops.set(@Settings, "CONFIG_NCS_SERVER_IP", NovellUtils.GetIPAddr)
          Ops.set(
            @Settings,
            "CONFIG_NCS_SBD_SIZE",
            Misc.SysconfigRead(path(".sysconfig.ncs.CONFIG_NCS_SBD_SIZE"), "8")
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_USE_MAX_SIZE",
            Misc.SysconfigRead(
              path(".sysconfig.ncs.CONFIG_NCS_USE_MAX_SIZE"),
              "no"
            )
          ) # we are doing a reconfigure
        else
          Builtins.y2milestone("Ncs.Read:we are in reconfigure read() section.")
          Ops.set(
            @Settings,
            "CONFIG_NCS_LDAP_SERVERS",
            Builtins.splitstring(
              Convert.to_string(
                SCR.Read(path(".sysconfig.ncs.CONFIG_NCS_LDAP_SERVERS"))
              ),
              ","
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_START",
            Misc.SysconfigRead(path(".sysconfig.ncs.CONFIG_NCS_START"), "Now")
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_CONFIG_TYPE",
            Misc.SysconfigRead(
              path(".sysconfig.ncs.CONFIG_NCS_CONFIG_TYPE"),
              "Existing Cluster"
            )
          )

          Ops.set(
            @Settings,
            "CONFIG_NCS_PROXY_USER",
            Misc.SysconfigRead(path(".sysconfig.ncs.CONFIG_NCS_PROXY_USER"), "")
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_USE_COMMON_PROXY",
            Misc.SysconfigRead(
              path(".sysconfig.ncs.CONFIG_NCS_USE_COMMON_PROXY"),
              "no"
            )
          )

          if useCommonProxy == "yes"
            Ops.set(@Settings, "CONFIG_NCS_PROXY_PASSWORD", commonProxyPassword)
          else
            Ops.set(@Settings, "CONFIG_NCS_PROXY_PASSWORD", "")
          end

          productInfo = NovellAddonInstall.GetProductInfo("ncs")
          tempSettings = NovellAddonInstall.GetSysconfigMap(
            Ops.get_string(productInfo, "config_file", "")
          )
          temp = Misc.SysconfigRead(
            path(".sysconfig.ncs.CONFIG_NCS_CLUSTER_DN"),
            ""
          )
          temp = NovellUtils.Strip(temp, " \n \t")
          Ops.set(@Settings, "CONFIG_NCS_CLUSTER_DN", temp)
          if Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_DN", "") == nil ||
              Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_DN", "") == ""
            Ops.set(
              @Settings,
              "CONFIG_NCS_CLUSTER_DN",
              Ops.get_string(tempSettings, "CONFIG_NCS_CLUSTER_DN", "")
            )
          end

          if Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "New Cluster"
            Ops.set(
              @Settings,
              "CONFIG_NCS_CLUSTER_IP",
              Misc.SysconfigRead(
                path(".sysconfig.ncs.CONFIG_NCS_CLUSTER_IP"),
                ""
              )
            )
            Ops.set(
              @Settings,
              "CONFIG_NCS_SBD_DEV",
              Misc.SysconfigRead(path(".sysconfig.ncs.CONFIG_NCS_SBD_DEV"), "")
            )
            Ops.set(
              @Settings,
              "CONFIG_NCS_SBD_DEV2",
              Misc.SysconfigRead(path(".sysconfig.ncs.CONFIG_NCS_SBD_DEV2"), "")
            )
          end

          Ops.set(
            @Settings,
            "CONFIG_NCS_SERVER_NAME",
            Misc.SysconfigRead(
              path(".sysconfig.ncs.CONFIG_NCS_SERVER_NAME"),
              NovellUtils.GetHostname
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_SERVER_IP",
            Misc.SysconfigRead(
              path(".sysconfig.ncs.CONFIG_NCS_SERVER_IP"),
              NovellUtils.GetIPAddr
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_SBD_SIZE",
            Misc.SysconfigRead(path(".sysconfig.ncs.CONFIG_NCS_SBD_SIZE"), "8")
          )
          Ops.set(
            @Settings,
            "CONFIG_NCS_USE_MAX_SIZE",
            Misc.SysconfigRead(
              path(".sysconfig.ncs.CONFIG_NCS_USE_MAX_SIZE"),
              "no"
            )
          )
        end
      end

      Builtins.y2debug("NCS READ Complete:%1", @Settings)

      @globals_configured = true

      # Error message
      Report.Error(_("Cannot read current settings.")) if false

      # check dependencies
      return false if Abort()
      Progress.NextStage

      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end

    def ServerIsDsfwServer(ip)
      retCode = false

      cmd = Builtins.sformat(
        "/usr/bin/ldapsearch -x -H ldaps://%1:636 -s base -b \"\" | grep 2.16.840.1.113719.1.513.7.1 > /dev/null",
        ip
      )
      Builtins.y2milestone("Ncs.ServerIsDsfwServer:executing %1", cmd)
      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      retCode = true if Ops.get_integer(ret, "exit", -1) == 0 # server is a dsfw server

      Builtins.y2milestone(
        "Ncs.ServerIsDsfwServer:returned %1",
        Ops.get_integer(ret, "exit", -1)
      )
      retCode
    end

    # Write all ncs settings
    # @return true on success
    def Write
      pattern = "novell-ncs"
      Wizard.DisableAbortButton
      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end

        return true
      end
      if Convert.to_string(SCR.Read(path(".sysconfig.ncs.SERVICE_CONFIGURED"))) == "yes"
        return true
      end

      Builtins.y2milestone("Ncs::Write started")
      if !Ops.get(NovellAddonInstall.products_to_configure, "ncs", true)
        Builtins.y2milestone("NCS -- configure later")
        return true
      end

      abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if abortInstall == "yes"
        Builtins.y2milestone("Ncs:setting ABORT_OES_INSTALL is set to 'yes'.")
        return true
      end

      # See if XAD is installed.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "Ncs:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end
      Builtins.y2milestone("Ncs:xadMode:%1", @xadMode)


      if  !@upgradeModeOES11SP2 &&
          !@upgradeModeOES2015 &&
          !@upgradeModeOES2015SP1 &&
          !@upgradeModeOES11SP3 &&
          !@upgradeModeOES2018 &&
          !@upgradeModeOES2018SP1 &&
          !@upgradeModeOES2018SP2 
        # If Aquila did not install properly, then we don't want to proceed at this point.
        Builtins.y2milestone(
          "Ncs:testing to see if eDir configured properly. Xad mode: %1",
          @xadMode
        )
        if @xadMode == true
          Builtins.y2milestone(
            "Ncs:eDir service_configured (mem):%1",
            Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "")
          )
          Builtins.y2milestone(
            "Ncs:eDir service_configured (file):%1",
            Convert.to_string(
              SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
            )
          )
          if Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "") != "yes" &&
              Convert.to_string(
                SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
              ) != "yes"
            if EdirUtils.EdirConfigProblemPrompt("OES Cluster Services")
              Builtins.y2milestone(
                "Ncs:aborting NCS write by user choice since eDirectory was not configured."
              )
              return true
            end
          end
        end

        # Ncs write dialog caption
        caption = _("Saving OES Cluster Services (NCS) Configuration")

        stages = [
          # Progress stage 1/9
          _("Write OES Cluster Services (NCS) settings"),
          # Progress stage 2/9
          _("Extend directory schema"),
          # Progress stage 3/9
          _("Write cluster library configuration file"),
          # Progress stage 4/9
          _("Write node name file"),
          # Progress stage 5/9
          _("Create cluster object in the directory"),
          # Progress stage 6/9
          _("Add node to cluster"),
          # Progress stage 7/9
          _("Create storage device"),
          # Progress stage 8/9
          _("Start OES Cluster Services (NCS)"),
          # Progress stage 9/9
          _("Finalize OES Cluster Services (NCS) settings")
        ]

        steps = Builtins.size(stages)

        # We do not set help text here, because it was set outside
        Progress.New(
          caption,
          _("Configure OES Cluster Services (NCS)"),
          steps,
          stages,
          stages,
          ""
        )

        Progress.NextStage

        # determine IP addresses
        server_list = Ops.get_list(
          NovellLDAP.Settings,
          "CONFIG_LDAP_SERVERS_LIST",
          []
        )
        primary_server = NovellLDAP.GetAddressServer(
          Ops.get(Ops.get_list(@Settings, "CONFIG_NCS_LDAP_SERVERS", []), 0, ""),
          server_list
        )
        fallback_list = NovellLDAP.GetFallbackServerList(
          Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
          server_list
        )
        Builtins.y2milestone("Primary server: %1", primary_server)
        Builtins.y2milestone("Fallback server_list: %1", fallback_list)

        full_admin_context = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        admin_pwd = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )
        if UseXadLdapForTreeAdmin() == true
          full_admin_context = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_CONTEXT", "")
          )
          admin_pwd = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_PASSWORD",
            ""
          )
        end

        proxyUser = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_NCS_PROXY_USER", "")
        )
        Builtins.y2milestone("Ncs.Write:CONFIG_NCS_PROXY_USER is %1", proxyUser)
        proxyPassword = Ops.get_string(
          @Settings,
          "CONFIG_NCS_PROXY_PASSWORD",
          ""
        )

        useCommonProxy = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_DO_PROXY",
          ""
        )
        Builtins.y2milestone("Ncs.Write:useCommonProxy: %1", useCommonProxy)
        if useCommonProxy == "yes"
          commonProxyName = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
          commonProxyPassword = NovellUtils.RetrieveProxyCredentials("password")
          if commonProxyPassword == nil || commonProxyPassword == ""
            commonProxyPassword = Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_PROXY_PASSWORD",
              ""
            )
          end

          if NovellUtils.Nds2Ldap(
              Ops.get_string(@Settings, "CONFIG_NCS_PROXY_USER", "")
            ) == commonProxyName
            proxyPassword = commonProxyPassword
            Ops.set(@Settings, "CONFIG_NCS_PROXY_PASSWORD", proxyPassword)
          end
        end

        if proxyUser == nil || proxyUser == "" || proxyPassword == nil ||
            proxyPassword == ""
          Builtins.y2milestone(
            "Ncs.Write: assigning ldap admin and password to proxy since a value is empty."
          )
          proxyUser = full_admin_context
          proxyPassword = admin_pwd
        end

        configType = Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "")
        serverName = Ops.get_string(@Settings, "CONFIG_NCS_SERVER_NAME", "")
        serverIp = Ops.get_string(@Settings, "CONFIG_NCS_SERVER_IP", "")
        clusterDn = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_DN", "")
        )
        if Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV", "") == nil
          Ops.set(@Settings, "CONFIG_NCS_SBD_DEV", "")
        end # avoid nil value
        if Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV2", "") == nil
          Ops.set(@Settings, "CONFIG_NCS_SBD_DEV2", "")
        end

        #write the /etc/sysconfig file
        SCR.Write(path(".sysconfig.ncs.SERVICE_CONFIGURED"), "no")

        # Flush the file to disk for the python.
        SCR.Write(path(".sysconfig.ncs"), nil)

        if configType != "Configure Later"
          #apply settings

          # Since we do not want to write the password in clear text, we will write the password
          # to the temp dir that only root is allowed to see.  It is autodeleted by the system.
          # The python ncs code will then read from this file and get the data.
          tmpdir = Convert.to_string(SCR.Read(path(".target.tmpdir")))

          if tmpdir == nil || tmpdir == ""
            Builtins.y2security("Using /tmp directory!")
            tmpdir = "/tmp"
            Builtins.y2milestone("Ncs:setting tempDir to: %1\n", tmpdir)
          end

          # Now that we have the system tmp directory, write it to a file at a well known location.
          SCR.Execute(path(".target.mkdir"), "/var/opt/novell/install")
          bval = SCR.Write(
            path(".target.string"),
            "/var/opt/novell/install/tmpdir",
            tmpdir
          )
          Report.Warning(_("Could not create tmpdir file.")) if bval != true

          # Now write the LDAP password file to the protected tmp dir.
          tmp = Ops.add(tmpdir, "/ncsfile")
          Builtins.y2milestone("Ncs:tempDir file: %1\n", tmp)
          SCR.Write(path(".target.string"), tmp, admin_pwd)

          ldap_servers_info = ""
          local_ldap_servers_info = ""
          serverIsDsfw = false

          Builtins.foreach(
            Ops.get_list(@Settings, "CONFIG_NCS_LDAP_SERVERS", [])
          ) do |server|
            server_info = NovellLDAP.GetAddressServer(server, server_list)
            serverLocal = false
            local_server_list = NovellLDAP.GetLocalServers(nil)
            Builtins.foreach(local_server_list) do |loc_server_info|
              if Ops.get_string(loc_server_info, "CONFIG_LDAP_SERVER_IP", "") ==
                  Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
                serverLocal = true
              end
            end
            serverIsDsfw = ServerIsDsfwServer(
              Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
            )
            if serverLocal
              local_ldap_servers_info = Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(local_ldap_servers_info, ",ldaps://"),
                    Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
                  ),
                  ":"
                ),
                serverIsDsfw ?
                  "1636" :
                  Builtins.sformat(
                    "%1",
                    Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636)
                  )
              )
            else
              ldap_servers_info = Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(ldap_servers_info, ",ldaps://"),
                    Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
                  ),
                  ":"
                ),
                serverIsDsfw ?
                  "1636" :
                  Builtins.sformat(
                    "%1",
                    Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636)
                  )
              )
            end
          end 

          ldap_servers_info = Ops.add(
            local_ldap_servers_info,
            ldap_servers_info
          )
          ldap_servers_info = Builtins.substring(ldap_servers_info, 1) # remove the "," from the front

          # create the settings file
          settings_string = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              Ops.add(
                                Ops.add(
                                  Ops.add(
                                    Ops.add(
                                      Ops.add(
                                        Ops.add(
                                          Ops.add(
                                            Ops.add(
                                              Ops.add(
                                                Ops.add(
                                                  "CONFIG_NCS_CONFIG_TYPE=",
                                                  Ops.get_string(
                                                    @Settings,
                                                    "CONFIG_NCS_CONFIG_TYPE",
                                                    ""
                                                  )
                                                ),
                                                "\n"
                                              ),
                                              "CONFIG_NCS_CLUSTER_DN="
                                            ),
                                            Ops.get_string(
                                              @Settings,
                                              "CONFIG_NCS_CLUSTER_DN",
                                              ""
                                            )
                                          ),
                                          "\n"
                                        ),
                                        "CONFIG_NCS_START="
                                      ),
                                      Ops.get_string(
                                        @Settings,
                                        "CONFIG_NCS_START",
                                        "Now"
                                      )
                                    ),
                                    "\n"
                                  ),
                                  "CONFIG_NCS_SERVER_NAME="
                                ),
                                Ops.get_string(
                                  @Settings,
                                  "CONFIG_NCS_SERVER_NAME",
                                  ""
                                )
                              ),
                              "\n"
                            ),
                            "CONFIG_NCS_SERVER_IP="
                          ),
                          Ops.get_string(@Settings, "CONFIG_NCS_SERVER_IP", "")
                        ),
                        "\n"
                      ),
                      "CONFIG_NCS_ADMIN_DN="
                    ),
                    full_admin_context
                  ),
                  "\n"
                ),
                "CONFIG_NCS_LDAP_INFO="
              ),
              ldap_servers_info
            ),
            "\n"
          )

          if Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "New Cluster"
            settings_string = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              Ops.add(
                                Ops.add(
                                  Ops.add(
                                    settings_string,
                                    "CONFIG_NCS_CLUSTER_IP="
                                  ),
                                  Ops.get_string(
                                    @Settings,
                                    "CONFIG_NCS_CLUSTER_IP",
                                    ""
                                  )
                                ),
                                "\n"
                              ),
                              "CONFIG_NCS_SBD_DEV="
                            ),
                            Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV", "")
                          ),
                          "\n"
                        ),
                        "CONFIG_NCS_SBD_DEV2="
                      ),
                      Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV2", "")
                    ),
                    "\n"
                  ),
                  "CONFIG_NCS_SBD_SIZE="
                ),
                Ops.get_string(@Settings, "CONFIG_NCS_SBD_SIZE", "")
              ),
              "\n"
            )
          end
          Builtins.y2milestone(
            "Ncs.Write:settings file entries:\n%1",
            settings_string
          )

          # create the proxy settings file
          proxy_settings_string = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              Ops.add(
                                Ops.add(
                                  Ops.add(
                                    Ops.add(
                                      Ops.add(
                                        Ops.add(
                                          Ops.add(
                                            Ops.add(
                                              Ops.add(
                                                Ops.add(
                                                  Ops.add(
                                                    Ops.add(
                                                      Ops.add(
                                                        "CONFIG_NCS_CONFIG_TYPE=",
                                                        Ops.get_string(
                                                          @Settings,
                                                          "CONFIG_NCS_CONFIG_TYPE",
                                                          ""
                                                        )
                                                      ),
                                                      "\n"
                                                    ),
                                                    "CONFIG_NCS_CLUSTER_DN="
                                                  ),
                                                  Ops.get_string(
                                                    @Settings,
                                                    "CONFIG_NCS_CLUSTER_DN",
                                                    ""
                                                  )
                                                ),
                                                "\n"
                                              ),
                                              "CONFIG_NCS_START="
                                            ),
                                            Ops.get_string(
                                              @Settings,
                                              "CONFIG_NCS_START",
                                              "Now"
                                            )
                                          ),
                                          "\n"
                                        ),
                                        "CONFIG_NCS_SERVER_NAME="
                                      ),
                                      Ops.get_string(
                                        @Settings,
                                        "CONFIG_NCS_SERVER_NAME",
                                        ""
                                      )
                                    ),
                                    "\n"
                                  ),
                                  "CONFIG_NCS_SERVER_IP="
                                ),
                                Ops.get_string(
                                  @Settings,
                                  "CONFIG_NCS_SERVER_IP",
                                  ""
                                )
                              ),
                              "\n"
                            ),
                            "CONFIG_NCS_ADMIN_DN="
                          ),
                          full_admin_context
                        ),
                        "\n"
                      ),
                      "CONFIG_NCS_MGT_USERNAME="
                    ),
                    proxyUser
                  ),
                  "\n"
                ),
                "CONFIG_NCS_LDAP_INFO="
              ),
              ldap_servers_info
            ),
            "\n"
          )

          if Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "New Cluster"
            proxy_settings_string = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              Ops.add(
                                Ops.add(
                                  Ops.add(
                                    proxy_settings_string,
                                    "CONFIG_NCS_CLUSTER_IP="
                                  ),
                                  Ops.get_string(
                                    @Settings,
                                    "CONFIG_NCS_CLUSTER_IP",
                                    ""
                                  )
                                ),
                                "\n"
                              ),
                              "CONFIG_NCS_SBD_DEV="
                            ),
                            Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV", "")
                          ),
                          "\n"
                        ),
                        "CONFIG_NCS_SBD_DEV2="
                      ),
                      Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV2", "")
                    ),
                    "\n"
                  ),
                  "CONFIG_NCS_SBD_SIZE="
                ),
                Ops.get_string(@Settings, "CONFIG_NCS_SBD_SIZE", "")
              ),
              "\n"
            )
          end
          Builtins.y2milestone(
            "Ncs.Write:proxy settings file entries:\n%1",
            proxy_settings_string
          )

          if @testMode
            Builtins.y2milestone(
              "mtc:settings file entries:\n" +
                "%1\n" +
                "Proxy Password: %2",
              settings_string,
              proxyPassword
            )
          end

          SCR.Write(
            path(".target.string"),
            Ops.add(tmpdir, "/settings"),
            settings_string
          )

          return false if Abort()
          Progress.NextStage

          cmd = ""
          rmap = {}

          # extend the schema
          Builtins.y2milestone("NCS: extending schema")
          cmd = Ops.add(
            Ops.add("/opt/novell/ncs/install/ncs_install.py -e -f ", tmpdir),
            "/settings"
          )
          rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
          if Ops.get_integer(rmap, "exit", -1) != 0
            Builtins.y2error(
              "NCS ncs_install.py -e failed:%1",
              Ops.get_string(rmap, "stderr", "")
            )
            Report.Error(_("NCS install failed to extend schema."))
            Builtins.y2milestone(
              "NCS:Sent up an error report:%1",
              Ops.get_integer(rmap, "exit", -1)
            )

            # Clean up
            cmd = Ops.add(
              Ops.add("/opt/novell/ncs/install/ncs_install.py -r -f ", tmpdir),
              "/settings"
            )
            rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
            Builtins.y2milestone("NCS:Cleanup return: %1", rmap)

            return false if @testMode == false
          end

          return false if Abort()
          Progress.NextStage

          # create clstrlib.conf
          Builtins.y2milestone("NCS: creating clstrlib.conf")
          cmd = Ops.add(
            Ops.add("/opt/novell/ncs/install/ncs_install.py -l -f ", tmpdir),
            "/settings"
          )
          rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
          if Ops.get_integer(rmap, "exit", -1) != 0
            Builtins.y2error(
              "NCS ncs_install.py -l failed:%1",
              Ops.get_string(rmap, "stderr", "")
            )
            Report.Error(_("NCS install failed to create clstrlib.conf file."))
            Builtins.y2milestone(
              "NCS:Sent up an error report:%1",
              Ops.get_integer(rmap, "exit", -1)
            )

            # Clean up
            cmd = Ops.add(
              Ops.add("/opt/novell/ncs/install/ncs_install.py -r -f ", tmpdir),
              "/settings"
            )
            rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
            Builtins.y2milestone("NCS:Cleanup return: %1", rmap)

            return false if @testMode == false
          end

          return false if Abort()
          Progress.NextStage

          # create nodename file
          Builtins.y2milestone("NCS: creating node name file")
          cmd = Ops.add(
            Ops.add("/opt/novell/ncs/install/ncs_install.py -n -f ", tmpdir),
            "/settings"
          )
          rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
          if Ops.get_integer(rmap, "exit", -1) != 0
            Builtins.y2error(
              "NCS ncs_install.py -n failed:%1",
              Ops.get_string(rmap, "stderr", "")
            )
            Report.Error(_("NCS install failed to create node name file."))
            Builtins.y2milestone(
              "NCS:Sent up an error report:%1",
              Ops.get_integer(rmap, "exit", -1)
            )

            # Clean up
            cmd = Ops.add(
              Ops.add("/opt/novell/ncs/install/ncs_install.py -r -f ", tmpdir),
              "/settings"
            )
            rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
            Builtins.y2milestone("NCS:Cleanup return: %1", rmap)

            return false if @testMode == false
          end

          return false if Abort()
          Progress.NextStage

          if !@upgradeModeOES11SP2 &&
              !@upgradeModeOES2015 &&
              !@upgradeModeOES2015SP1 &&
              !@upgradeModeOES11SP3 &&
              !@upgradeModeOES2018 &&
              !@upgradeModeOES2018SP1 &&
              !@upgradeModeOES2018SP2 
            # create cluster object
            Builtins.y2milestone("NCS: creating cluster object")
            cmd = Ops.add(
              Ops.add("/opt/novell/ncs/install/ncs_install.py -c -f ", tmpdir),
              "/settings"
            )
            rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
            if Ops.get_integer(rmap, "exit", -1) != 0
              Builtins.y2error(
                "NCS ncs_install.py -c failed:%1",
                Ops.get_string(rmap, "stderr", "")
              )
              Report.Error(_("NCS install failed to create cluster object."))
              Builtins.y2milestone(
                "NCS:Sent up an error report:%1",
                Ops.get_integer(rmap, "exit", -1)
              )

              # Clean up
              cmd = Ops.add(
                Ops.add("/opt/novell/ncs/install/ncs_install.py -r -f ", tmpdir),
                "/settings"
              )
              rmap = Convert.to_map(
                SCR.Execute(path(".target.bash_output"), cmd)
              )
              Builtins.y2milestone("NCS:Cleanup return: %1", rmap)

              return false if @testMode == false
            end
          end

          return false if Abort()
          Progress.NextStage

          if !@upgradeModeOES11SP2 &&
              !@upgradeModeOES2015 &&
              !@upgradeModeOES2015SP1 &&
              !@upgradeModeOES11SP3 &&
              !@upgradeModeOES2018 &&
              !@upgradeModeOES2018SP1 &&
              !@upgradeModeOES2018SP2 
            # add node to cluster
            Builtins.y2milestone("NCS: adding node to cluster")
            cmd = Ops.add(
              Ops.add("/opt/novell/ncs/install/ncs_install.py -a -f ", tmpdir),
              "/settings"
            )
            rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
            if Ops.get_integer(rmap, "exit", -1) != 0
              Builtins.y2error(
                "NCS ncs_install.py -a failed:%1",
                Ops.get_string(rmap, "stderr", "")
              )
              Report.Error(_("NCS install failed to add node to cluster."))
              Builtins.y2milestone(
                "NCS:Sent up an error report:%1",
                Ops.get_integer(rmap, "exit", -1)
              )

              # Clean up
              cmd = Ops.add(
                Ops.add("/opt/novell/ncs/install/ncs_install.py -r -f ", tmpdir),
                "/settings"
              )
              rmap = Convert.to_map(
                SCR.Execute(path(".target.bash_output"), cmd)
              )
              Builtins.y2milestone("NCS:Cleanup return: %1", rmap)

              return false if @testMode == false
            end
          end

          return false if Abort()
          Progress.NextStage

          if !@upgradeModeOES11SP2 &&
              !@upgradeModeOES2015 &&
              !@upgradeModeOES2015SP1 &&
              !@upgradeModeOES11SP3 &&
              !@upgradeModeOES2018 &&
              !@upgradeModeOES2018SP1 &&
              !@upgradeModeOES2018SP2

            # create sbd
            Builtins.y2milestone("NCS: creating SBD")
            cmd = Ops.add(
              Ops.add("/opt/novell/ncs/install/ncs_install.py -b -f ", tmpdir),
              "/settings"
            )
            rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
            if Ops.get_integer(rmap, "exit", -1) != 0
              Builtins.y2error(
                "NCS ncs_install.py -b failed:%1",
                Ops.get_string(rmap, "stderr", "")
              )
              Report.Error(_("NCS install failed to create SBD."))
              Builtins.y2milestone(
                "NCS:Sent up an error report:%1",
                Ops.get_integer(rmap, "exit", -1)
              )

              # Clean up
              cmd = Ops.add(
                Ops.add("/opt/novell/ncs/install/ncs_install.py -r -f ", tmpdir),
                "/settings"
              )
              rmap = Convert.to_map(
                SCR.Execute(path(".target.bash_output"), cmd)
              )
              Builtins.y2milestone("NCS:Cleanup return: %1", rmap)

              return false if @testMode == false
            end
          end

          # This last call sets the proxy user
          if !@upgradeModeOES11SP2 &&
	      !@upgradeModeOES2015 &&
              !@upgradeModeOES2015SP1 &&
              !@upgradeModeOES11SP3 &&
              !@upgradeModeOES2018 &&
              !@upgradeModeOES2018SP1 &&
              !@upgradeModeOES2018SP2

            tmpStr = Ops.add(tmpdir, "/ncsfile")
            passwords = Ops.add(Ops.add(admin_pwd, "\n"), proxyPassword)

            Builtins.y2milestone("Ncs:tempDir file: %1\n", tmpStr)
            SCR.Write(path(".target.string"), tmpStr, passwords)
            SCR.Write(
              path(".target.string"),
              Ops.add(tmpdir, "/settings"),
              proxy_settings_string
            )

            # send proxy user down
            Builtins.y2milestone("NCS: setting proxy user")
            cmd = Ops.add(
              Ops.add(
                Ops.add("/opt/novell/ncs/install/ncs_install.py -l -f ", tmpdir),
                "/settings"
              ),
              " -u "
            )
            rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
            if Ops.get_integer(rmap, "exit", -1) != 0
              Builtins.y2error(
                "NCS ncs_install.py -b failed:%1",
                Ops.get_string(rmap, "stderr", "")
              )
              Report.Error(_("NCS install failed to set proxy user."))
              Builtins.y2milestone(
                "NCS:Sent up an error report:%1",
                Ops.get_integer(rmap, "exit", -1)
              )

              # Clean up
              cmd = Ops.add(
                Ops.add("/opt/novell/ncs/install/ncs_install.py -r -f ", tmpdir),
                "/settings"
              )
              rmap = Convert.to_map(
                SCR.Execute(path(".target.bash_output"), cmd)
              )
              Builtins.y2milestone("NCS:Cleanup return: %1", rmap)

              return false if @testMode == false
            end
          end

          return false if Abort()
          Progress.NextStage

          # open protocol 224 for NCS
          Builtins.y2milestone("NCS: opening firewall protocol 224")
          retVal = false
          broadcast = { "EXT" => ["yes"] }
          if SuSEFirewall.Read
            retVal = SuSEFirewall.AddService("224", "IP", "EXT")
            SuSEFirewall.SetBroadcastAllowedPorts(broadcast)
          end
          retVal = SuSEFirewall.WriteConfiguration if retVal

          #in case this is a post install we need to make sure that the firewall gets restarted
          if Mode.normal || Mode.installation && Stage.normal
            Builtins.y2milestone("NCS: restarting firewall")
            SuSEFirewall.ActivateConfiguration
          end

          # start ncs
          StartNCS()
        else
          return false if Abort()
          Progress.NextStage
          Progress.NextStage
          Progress.NextStage
          Progress.NextStage
          Progress.NextStage
          Progress.NextStage
          Progress.NextStage
        end # We are doing an upgrade
      else
        # open protocol 224 for NCS
        Builtins.y2milestone("NCS: opening firewall protocol 224 and 7023")

        retVal = false

        retVal = NovellUtils.SetFirewallPort(7023, :tcp, true)
        retVal = NovellUtils.SetFirewallPort(224, :udp, true)

        broadcast = { "EXT" => ["yes"] }
        if SuSEFirewall.Read
          retVal = SuSEFirewall.AddService("224", "IP", "EXT")
          SuSEFirewall.SetBroadcastAllowedPorts(broadcast)
        end
        retVal = SuSEFirewall.WriteConfiguration if retVal

        Builtins.y2milestone("NCS: restarting firewall")
        SuSEFirewall.ActivateConfiguration


        if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1 || @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 || @upgradeModeOES2018SP2

          ncs_proxyuser = NovellUtils.Nds2Ldap(Ops.get_string(@Settings, "CONFIG_NCS_PROXY_USER", ""))
          proxyUser = NovellUtils.Nds2Ldap(
             Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          if ncs_proxyuser == proxyUser  
            Builtins.y2milestone("Ncs:Setting commonproxy username and password during upgrade")
            proxyPassword = NovellUtils.RetrieveProxyCredentials("password")
            ret = NovellUtils.StoreCredsInOCSforUpgrade(proxyUser, proxyPassword,"NovellClusterServices.Novell")
            if ret != 0
              Popup.Message(
              Builtins.sformat(_("Error storing credentials in OCS for Common Proxy User: %1"), ret ))
              Builtins.y2milestone(
              "ncs.ConfigureProxyUser:Error storing credentials in OCS for common Proxy User: %1", ret)
	    else
              Ops.set(@Settings, "CONFIG_NCS_USE_COMMON_PROXY", "yes")
            end
          else
            Builtins.y2milestone("NovellNcs:NCS using service proxy, moving to common proxy")
            NovellUtils.MovetoCommonProxyDuringUpgrade("novell-ncs")
            Ops.set(@Settings, "CONFIG_NCS_PROXY_USER", Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          end
        end
        if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1 || @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 || @upgradeModeOES2018SP2
          ncs_proxyuser = NovellUtils.Nds2Ldap(Ops.get_string(@Settings, "CONFIG_NCS_PROXY_USER", ""))
          proxyUser = NovellUtils.Nds2Ldap(
             Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))

          if ncs_proxyuser != proxyUser
            Builtins.y2milestone("NovellNcs:NCS using service proxy, moving to common proxy")
            NovellUtils.MovetoCommonProxyDuringUpgrade("novell-ncs")
            Ops.set(@Settings, "CONFIG_NCS_PROXY_USER", Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          end
        end

        services = ["novell-ncs"]
        if Ops.get_string(@Settings, "CONFIG_NCS_START", "Now") == "Now"
          Builtins.foreach(services) do |service|
            if Service.Enabled(service)
              Service.Stop(service)
            else
              Builtins.y2milestone("%1 service is not enabled",service)
            end
          end
          services = ["novell-ncs"]
          Builtins.foreach(services) do |service|
            if Service.Enabled(service)
            Service.Start(service)
              if Service.Active(service)
                Builtins.y2milestone("Started %1 service",service)
              else
                Builtins.y2milestone("Starting %l service failed",service)
              end
            end
          end
        end
      end

      cmd = "/sbin/lvmconf --enable-cluster"
     
      Builtins.y2milestone("Ncs:executing %1", cmd)
      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      Builtins.y2milestone("Output: %1",ret)

      return false if Abort()
      Progress.NextStage

      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_LDAP_SERVERS"),
        Builtins.mergestring(
          Ops.get_list(@Settings, "CONFIG_NCS_LDAP_SERVERS", []),
          ","
        )
      )
      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_CONFIG_TYPE"),
        Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "")
      )
      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_CLUSTER_DN"),
        Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_DN", "")
      )
      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_CONFIG_TYPE"),
        Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "")
      )
      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_START"),
        Ops.get_string(@Settings, "CONFIG_NCS_START", "Now")
      )

      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_USE_COMMON_PROXY"),
        Ops.get_string(@Settings, "CONFIG_NCS_USE_COMMON_PROXY", "")
      )
      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_PROXY_USER"),
        Ops.get_string(@Settings, "CONFIG_NCS_PROXY_USER", "")
      )

      if Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "New Cluster"
        SCR.Write(
          path(".sysconfig.ncs.CONFIG_NCS_CLUSTER_IP"),
          Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_IP", "")
        )
        SCR.Write(
          path(".sysconfig.ncs.CONFIG_NCS_SBD_DEV"),
          Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV", "")
        )
        SCR.Write(
          path(".sysconfig.ncs.CONFIG_NCS_SBD_DEV2"),
          Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV2", "")
        )
        SCR.Write(
          path(".sysconfig.ncs.CONFIG_NCS_SBD_SIZE"),
          Ops.get_string(@Settings, "CONFIG_NCS_SBD_SIZE", "")
        )
        SCR.Write(
          path(".sysconfig.ncs.CONFIG_NCS_USE_MAX_SIZE"),
          Ops.get_string(@Settings, "CONFIG_NCS_USE_MAX_SIZE", "")
        )
      else
        #set the values to empty
        #IS THERE A WAY TO REMOVE THESE KEYS FROM THE SYSCONFIG FILE???
        SCR.Write(path(".sysconfig.ncs.CONFIG_NCS_CLUSTER_IP"), "")
        SCR.Write(path(".sysconfig.ncs.CONFIG_NCS_SBD_DEV"), "")
        SCR.Write(path(".sysconfig.ncs.CONFIG_NCS_SBD_DEV2"), "")
      end

      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_SERVER_NAME"),
        Ops.get_string(@Settings, "CONFIG_NCS_SERVER_NAME", "")
      )
      SCR.Write(
        path(".sysconfig.ncs.CONFIG_NCS_SERVER_IP"),
        Ops.get_string(@Settings, "CONFIG_NCS_SERVER_IP", "")
      )

      SCR.Write(path(".sysconfig.ncs.SERVICE_CONFIGURED"), "yes")
      SCR.Write(path(".sysconfig.ncs"), nil)

      if Builtins.setenv("LDAPTLS_REQCERT", "")
        Builtins.y2milestone("LDAPTLS_REQCERT value is unset")
      else
        Builtins.y2milestone("Failed to unset TLSLDAP_REQCERT")
      end


      return false if Abort()
      # Progress finished
      Progress.Finish

      return false if Abort()
      true
    end

    # Get all ncs settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "ncs", false)
        return true
      end

      Ops.set(
        @Settings,
        "CONFIG_NCS_LDAP_SERVERS",
        Builtins.splitstring(Ops.get_string(settings, "ldap_servers", ""), ",")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_CONFIG_TYPE",
        Ops.get_string(settings, "config_type", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_START",
        Ops.get_string(settings, "start", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_CLUSTER_DN",
        Ops.get_string(settings, "cluster_dn", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_CLUSTER_IP",
        Ops.get_string(settings, "cluster_ip", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_SBD_DEV",
        Ops.get_string(settings, "sbd_dev", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_SBD_DEV2",
        Ops.get_string(settings, "sbd_dev2", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_SBD_SIZE",
        Ops.get_string(settings, "sbd_size", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_PROXY_USER",
        Ops.get_string(settings, "proxy_user", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NCS_PROXY_PASSWORD",
        Ops.get_string(settings, "proxy_user_password", "")
      )

      if Ops.get_string(settings, "server_name", "") == ""
        Ops.set(@Settings, "CONFIG_NCS_SERVER_NAME", NovellUtils.GetHostname)
      else
        Ops.set(
          @Settings,
          "CONFIG_NCS_SERVER_NAME",
          Ops.get_string(settings, "server_name", "")
        )
      end

      Ops.set(@Settings, "CONFIG_NCS_SERVER_IP", NovellUtils.GetIPAddr)

      @globals_configured = true
      true
    end

    # Dump the ncs settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-ncs") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "ncs", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "ncs", true)
        Ops.set(
          settings,
          "ldap_servers",
          Builtins.mergestring(
            Ops.get_list(@Settings, "CONFIG_NCS_LDAP_SERVERS", []),
            ","
          )
        )
        Ops.set(
          settings,
          "config_type",
          Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "")
        )
        Ops.set(
          settings,
          "start",
          Ops.get_string(@Settings, "CONFIG_NCS_START", "")
        )
        Ops.set(
          settings,
          "cluster_dn",
          Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_DN", "")
        )
        Ops.set(
          settings,
          "cluster_ip",
          Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_IP", "")
        )
        Ops.set(
          settings,
          "sbd_dev",
          Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV", "")
        )
        Ops.set(
          settings,
          "sbd_dev2",
          Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV2", "")
        )
        Ops.set(
          settings,
          "server_name",
          Ops.get_string(@Settings, "CONFIG_NCS_SERVER_NAME", "")
        )
        Ops.set(
          settings,
          "sbd_size",
          Ops.get_string(@Settings, "CONFIG_NCS_SBD_SIZE", "8")
        )
        Ops.set(
          settings,
          "proxy_user",
          Ops.get_string(@Settings, "CONFIG_NCS_PROXY_USER", "")
        )
        Ops.set(settings,"proxy_user_password",NovellUtils.passwordMessage)
      end

      deep_copy(settings)
    end

    # Tranlate a ncs start string
    # @return translated "Now" or "Later" string
    def Trans_ncs_start(ncsstart)
      if Builtins.tolower(ncsstart) == "later"
        return _("Later")
      else
        return _("Now")
      end
    end

    # Tranlate a ncs type string
    # @return translated "New Cluster" or "Existing Cluster" string
    def Trans_ncs_type(ncstype)
      if Builtins.tolower(ncstype) == "existing cluster"
        return _("Existing Cluster")
      else
        return _("New Cluster")
      end
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""
      if Convert.to_string(SCR.Read(path(".sysconfig.ncs.SERVICE_CONFIGURED"))) == "yes"
        summary = Summary.OpenList(summary)
        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          _("NCS is already configured, you cannot reconfigure it.")
        )

        summary = Summary.CloseList(summary)

        Builtins.y2milestone("NCS.Summary: Reconfigure mode.")
        return [summary, []]
      end

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        full_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_CONTEXT",
          ""
        )
        if UseXadLdapForTreeAdmin() == true
          full_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
        end
        summary = Summary.OpenList(summary)
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("New or Existing Cluster: %1"),
            Trans_ncs_type(
              Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("LDAP Servers: %1"),
            Builtins.mergestring(
              Ops.get_list(@Settings, "CONFIG_NCS_LDAP_SERVERS", []),
              ","
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(_("LDAP Username: %1"), full_admin_context)
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Cluster object in the directory: %1"),
            Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_DN", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("NCS Proxy User: %1"),
            Ops.get_string(@Settings, "CONFIG_NCS_PROXY_USER", "")
          )
        )
        #summary = Summary::AddListItem(summary, sformat(_("NCS Proxy User Password: %1"),Settings["CONFIG_NCS_PROXY_PASSWORD"]:""));

        if @autoyast_tool
          summary = Summary.AddListItem(summary, _("Node name: Local Host"))
          summary = Summary.AddListItem(
            summary,
            _("Node's IP address: Local Host")
          )
        else
          summary = Summary.AddListItem(
            summary,
            Builtins.sformat(
              _("Node name: %1"),
              Ops.get_string(@Settings, "CONFIG_NCS_SERVER_NAME", "")
            )
          )
          summary = Summary.AddListItem(
            summary,
            Builtins.sformat(
              _("Node's IP address / hostname: %1"),
              Ops.get_string(@Settings, "CONFIG_NCS_SERVER_IP", "")
            )
          )
        end
        if Ops.get_string(@Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "New Cluster"
          summary = Summary.AddListItem(
            summary,
            Builtins.sformat(
              _("Cluster IP address / hostname: %1"),
              Ops.get_string(@Settings, "CONFIG_NCS_CLUSTER_IP", "")
            )
          )
          summary = Summary.AddListItem(
            summary,
            Builtins.sformat(
              _("SBD device: %1"),
              Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV", "")
            )
          )
          summary = Summary.AddListItem(
            summary,
            Builtins.sformat(
              _("SBD mirror device: %1"),
              Ops.get_string(@Settings, "CONFIG_NCS_SBD_DEV2", "")
            )
          )
          if Ops.get_string(@Settings, "CONFIG_NCS_USE_MAX_SIZE", "") == "yes"
            summary = Summary.AddListItem(
              summary,
              Builtins.sformat(_("SBD size: %1"), "Maximum Size")
            )
          else
            summary = Summary.AddListItem(
              summary,
              Builtins.sformat(
                _("SBD size: %1"),
                Ops.get_string(@Settings, "CONFIG_NCS_SBD_SIZE", "")
              )
            )
          end
        end
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Start Cluster Services: %1"),
            Trans_ncs_start(
              Ops.get_string(@Settings, "CONFIG_NCS_START", "Now")
            )
          )
        )

        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      { "install" => [], "remove" => [] }
    end

    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :variable => :xad_packages_installed, :type => "boolean"
    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :variable => :testMode, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :edir_service_configured, :type => "boolean"
    publish :variable => :upgradeModeOES11SP2, :type => "boolean"
    publish :variable => :upgradeModeOES11SP3, :type => "boolean"
    publish :variable => :upgradeModeOES2015, :type => "boolean"
    publish :variable => :upgradeModeOES2015SP1, :type => "boolean"
    publish :variable => :upgradeModeOES2018, :type => "boolean"
    publish :variable => :upgradeModeOES2018SP1, :type => "boolean"
    publish :variable => :upgradeModeOES2018SP2, :type => "boolean"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :variable => :autoyast_tool, :type => "boolean"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :UseXadLdapForTreeAdmin, :type => "boolean ()"
    publish :function => :StartNCS, :type => "boolean ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :ServerIsDsfwServer, :type => "boolean (string)"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  Ncs = NcsClass.new
  Ncs.main
end
